--[[Copyright (C) Squigglze, 2025]]

--===============================================================
     -- Helicopter Mode For FS25 --
--===============================================================

HelicopterMode = {}
HelicopterMode._lastAltitude = 0

function HelicopterMode.prerequisitesPresent(specializations)
    return true
end

function HelicopterMode.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", HelicopterMode)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", HelicopterMode)
    SpecializationUtil.registerEventListener(vehicleType, "onDraw", HelicopterMode)
end

function HelicopterMode:onLoad(savegame)
    self.setHelicopterInput = HelicopterMode.setHelicopterInput

    self.heli_inputW = false
    self.heli_inputS = false
    self.heli_inputA = false
    self.heli_inputD = false
    self.heli_inputLMB = false
    self.heli_inputRMB = false

    self.lastInputW = false
    self.lastInputS = false
    self.lastInputA = false
    self.lastInputD = false
    self.lastInputLMB = false
    self.lastInputRMB = false

    self.heli_pitch = 0
    self.heli_sendPitch = 0
    self.heli_massResult = 1000
    self.heli_downForce = 0
    self.heli_yForce = 0

    setCenterOfMass(self.components[1].node, 0, -1, 0)

    HelicopterMode._instance = self
end

function HelicopterMode:setHelicopterInput(w, s, a, d, lmb, rmb)
    self.heli_inputW = w
    self.heli_inputS = s
    self.heli_inputA = a
    self.heli_inputD = d
    self.heli_inputLMB = lmb
    self.heli_inputRMB = rmb
end

function HelicopterMode:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    if not self:getIsControlled() then return end

    if self.isClient and self:getIsActiveForInput(true) then
        local w = Input.isKeyPressed(Input.KEY_w)
        local s = Input.isKeyPressed(Input.KEY_s)
        local a = Input.isKeyPressed(Input.KEY_a)
        local d = Input.isKeyPressed(Input.KEY_d)
        local lmb = Input.isMouseButtonPressed(Input.MOUSE_BUTTON_LEFT)
        local rmb = Input.isMouseButtonPressed(Input.MOUSE_BUTTON_RIGHT)

        if self.lastInputW ~= w or self.lastInputS ~= s or self.lastInputA ~= a or self.lastInputD ~= d
        or self.lastInputLMB ~= lmb or self.lastInputRMB ~= rmb then
            self.lastInputW = w
            self.lastInputS = s
            self.lastInputA = a
            self.lastInputD = d
            self.lastInputLMB = lmb
            self.lastInputRMB = rmb

            g_client:getServerConnection():sendEvent(HelicopterInputEvent.new(self, w, s, a, d, lmb, rmb))
        end
    end

    if not self.isServer then return end

    local node = self.components[1].node
    local mass = self:getTotalMass(true)
    self.heli_massResult = mass * 1000
    self.heli_downForce = mass * -9.81
    self.heli_yForce = self.heli_downForce * -0.75

    if self.heli_inputLMB then
        self.heli_pitch = math.min(self.heli_pitch + 3, 15)
    elseif self.heli_inputRMB then
        self.heli_pitch = math.max(self.heli_pitch - 3, -20)
    else
        if math.abs(self.heli_pitch) > 0.1 then
            self.heli_pitch = self.heli_pitch * 0.95
        else
            self.heli_pitch = 0
        end
    end
    local dX, _, dZ = localDirectionToWorld(node, 1, 0, 0)
    local sinDir = dX / 15
    local cosDir = dZ / -15
    local moveSpeed = 172.4 -- levegőbeli mozgási sebesség

    local axisForward = 0
    if self.heli_inputW then axisForward = 1 end
    if self.heli_inputS then axisForward = -1 end

    local zSteer = moveSpeed * axisForward
    local xForce = zSteer * cosDir
    local zForce = zSteer * sinDir

    -- Felhajtóerő pitch alapján
    self.heli_yForce = self.heli_pitch + (self.heli_downForce * -0.65)

    -- Alkalmazzuk az erőhatásokat
    addForce(node, xForce, self.heli_yForce, zForce, 0, 1.2, 0, true)

    -- Forgatás oldalirányú bemenet alapján
    local yTurn = 0
    if self.heli_inputA then
        yTurn = 1
    elseif self.heli_inputD then
        yTurn = -1
    end

    local AVx, AVy, AVz = getAngularVelocity(node)
    if yTurn == 0 then
        yTurn = AVy * 0.85
    end
    setAngularVelocity(node, AVx * 0.75, yTurn, AVz * 0.75)

    -- Stabilizálás: csak Y tengely forgás, a többi nulla
    setAngularVelocity(node, 0, yTurn, 0)

    -- Damping értékek a stabil mozgáshoz
    setLinearDamping(node, 0.15)
    setAngularDamping(node, 0.01)
end
function HelicopterMode:onDraw()
    if self.components ~= nil and self.components[1] ~= nil then
        local node = self.components[1].node
        local px, py, pz = getWorldTranslation(node)
        local groundY = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, px, 0, pz)
        local relativeHeight = math.max(0, py - groundY)
        local height = string.format("%.1f", relativeHeight)

        local LVx, LVy, LVz = getLinearVelocity(node)

        setTextColor(1, 1, 0, 1)
        setTextBold(false)
		setTextAlignment(RenderText.ALIGN_CENTER)
renderText(0.5, 0.05, 0.016, string.format(g_i18n:getText("heli_speed"), string.format("%.2f", math.sqrt(LVx^2 + LVy^2 + LVz^2))))
renderText(0.5, 0.03, 0.016, string.format(g_i18n:getText("heli_climb"), string.format("%.2f", LVy)))
renderText(0.5, 0.01, 0.016, string.format(g_i18n:getText("heli_altitude"), string.format("%.1f", height)))
        setTextColor(1, 1, 1, 1)
    end
end
function HelicopterMode.raycastCallback(hitObjectId, x, y, z, distance, nx, ny, nz, subShapeIndex, hitShapeId)
    if hitObjectId ~= 0 then
        local node = HelicopterMode._instance.components[1].node
        local _, py, _ = getWorldTranslation(node)
        HelicopterMode._lastAltitude = math.max(0, py - y)
    end
    return false
end
